'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _immutable = require('immutable');

var _EntityEditorConfig = require('./EntityEditorConfig');

var _EntityEditorConfig2 = _interopRequireDefault(_EntityEditorConfig);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var NO_HISTORY_ERROR_MESSAGE = 'Entity Editor: history prop must be passed to editor when using ReactRouter4Config';
/* eslint-disable no-unused-vars */

var NO_LOCATION_ERROR_MESSAGE = 'Entity Editor: location must be specified in actionProps when using ReactRouter4Config';

var go = function go(_ref) {
    var props = _ref.props;
    return function (_ref2) {
        var continueRouteChange = _ref2.continueRouteChange,
            location = _ref2.location;

        if (continueRouteChange) {
            continueRouteChange();
            return;
        }
        var history = props.history;

        if (!history) {
            throw new Error(NO_HISTORY_ERROR_MESSAGE);
        }
        if (!location) {
            throw new Error(NO_LOCATION_ERROR_MESSAGE);
        }
        history.push(location);
    };
};

function protectRouteChange(entityEditorInstance, config) {
    var ee = entityEditorInstance;
    var history = ee.nextProps.history;

    if (!history) {
        throw new Error(NO_HISTORY_ERROR_MESSAGE);
    }

    ee.unblockRouteChange && ee.unblockRouteChange();

    // create mutable actionProps container so we can pass it into a workflow
    // before we actually have its contents
    var actionProps = {};

    // when react-router is about to change routes, this function will be called
    // so we reject react-router's automatic route transition and instead
    // provide an identical one as an actionProp
    // action at the end of the "go" action / workflow
    ee.unblockRouteChange = history.block(function (nextLocation, action) {

        // if we're going back in history it would be great to warn against unsaved changes being lost
        // however the current mechanism causes react-router's history to call goBack multiple times.
        // to prevent this bug from surfacing for now, just don't protect against going back
        if (action == "POP") {
            return true;
        }

        // if current task is blocking, that means we're in the middle of an operation
        // and something has tried to change routes
        // we assume this route change was issued by the operation itself
        // and therefore do not block it
        if (ee.isCurrentTaskBlocking(ee.nextProps)) {
            return true;
        }

        // pass nextLocation to continueRouteChange, which returns a thunk
        var continueRouteChange = actionProps.continueRouteChange(nextLocation, action);
        // start the "go" action
        ee.workflowStart("go", config.getIn(["actions", "go"]), { continueRouteChange: continueRouteChange });
        return false;
    });

    actionProps.continueRouteChange = function (nextLocation, action) {
        return function () {
            ee.unblockRouteChange();
            switch (action) {
                case "PUSH":
                    history.push(nextLocation);
                    break;

                case "REPLACE":
                case "POP":
                    history.replace(nextLocation);
                    break;
            }
            protectRouteChange(entityEditorInstance, config);
        };
    };
}

function unprotectRouteChange(entityEditorInstance) {
    var ee = entityEditorInstance;
    ee.unblockRouteChange && ee.unblockRouteChange();
}

var ReactRouter4Config = (0, _EntityEditorConfig2.default)({
    operations: {
        go: go
    },
    lifecycleMethods: {
        componentWillMount: {
            reactRouter4: protectRouteChange
        },
        componentWillUnmount: {
            reactRouter4: unprotectRouteChange
        }
    }
});

exports.default = ReactRouter4Config;